rm(list=ls())

library(MASS)
library(tseries)
library(ccgarch)
library(MCMCpack)
library(rootSolve)
library(GGally)

### Read data ###

y.orig<-read.table("tsay_106.txt")
n<-dim(y.orig)[1]
m<-dim(y.orig)[2]
y.mean<-apply(y.orig,2,mean)
y<-matrix(ncol=m,nrow=n)
for (i in 1:n){
for (j in 1:m){
y[i,j]<-y.orig[i,j]-y.mean[j]
}}
colnames(y)<-colnames(y.orig)


#### Ordering the stocks according to the BIC ####

bic<-rep(NA,dim(y)[2])
pos<-1:dim(y)[2]
ccc<-combn(dim(y)[2],dim(y)[2]-1)
for (i in 1:dim(ccc)[2]){
test.mod<-lm(y[,-ccc[,i]]~y[,ccc[,i]]-1)
bic[pos[-ccc[,i]]]<-BIC(test.mod)
}
y.ord<-y[,order(bic,decreasing = TRUE)]
y<-y.ord
colnames(y)


### Time series plots & scatterplots ###

postscript("tsplot.eps")
par(mfrow=c(3,1))
plot(y[,1],type='l',xlab='time',ylab='cisco')
plot(y[,2],type='l',xlab='time',ylab='intel')
plot(y[,3],type='l',xlab='time',ylab='sp')
dev.off()

postscript("scatter.eps")
par(mfrow=c(3,1))
plot(y[,1],y[,2],xlab='cisco',ylab='intel')
plot(y[,1],y[,3],xlab='cisco',ylab='sp')
plot(y[,2],y[,3],xlab='intel',ylab='sp')
dev.off()

#### Log-GARCH fit provided by Christian Francq ####

VarAsymp<- function(omega,alpha.plus,alpha.moins,beta,eps,sig2init,petit,r0){
         n <- length(eps)
         derlogsigma2<-matrix(0,nrow=4,ncol=n)

log.sig2<-rep(0,n)
log.sig2[1]<-log(sig2init)
derlogsigma2[1:4,1]<-0
for(t in 2:n){
vec<-c(1,0,0,0)
log.sig2[t]<-omega+beta*log.sig2[t-1]
if(eps[t-1]> petit){
log.sig2[t]<-log.sig2[t]+alpha.plus*log(eps[t-1]^2)
vec[2]<-log(eps[t-1]^2)
}
if(eps[t-1]< -petit){
log.sig2[t]<-log.sig2[t]+alpha.moins*log(eps[t-1]^2)
vec[3]<-log(eps[t-1]^2)
}
vec[4]<-log.sig2[t-1]
derlogsigma2[1:4,t]<-vec+beta*derlogsigma2[1:4,(t-1)]
}
sig2<-exp(log.sig2[(r0+1):n])
eta <- eps[(r0+1):n]/sqrt(sig2)
eta <- eta/sd(eta)

J<-derlogsigma2[1:4,(r0+1):n]%*%t(derlogsigma2[1:4,(r0+1):n])/(n-r0)
kappa4<-mean(eta^4)

{if(kappa(J)<1/petit) inv<-solve(J) else inv<-matrix(0,nrow=4,ncol=4)}
var<-(kappa4-1)*inv
list(var=var,residus=eta,log.sig2=log.sig2)
}


objf.loggarch <- function(vartheta,eps,n,sig2init,eps.plus,eps.moins,r0){     
         omega <- vartheta[1]
         alpha.plus <- vartheta[2]
         alpha.moins <- vartheta[3]
         beta <- vartheta[4]
log.sig2<-rep(0,n)
log.sig2[1]<-log(sig2init)
for(t in 2:n){
log.sig2[t]<-omega+beta*log.sig2[t-1]+alpha.plus*eps.plus[t-1]+alpha.moins*eps.moins[t-1]}
#sig2<-pmax(exp(log.sig2[(r0+1):n]),petit)
sig2<-exp(log.sig2[(r0+1):n])
qml <- mean(eps[(r0+1):n]^2/sig2+log(sig2))
#if(qml=="NaN")
#write(file="bug.txt",vartheta,append=TRUE)
#write(file="bug.txt",qml,append=TRUE)
#if(qml=="NaN")write(file="bug.txt",sig2,append=TRUE)

qml }
#eps<-ser

estimLogGARCH<- function(omega,alpha.plus,alpha.moins,beta,eps,factor,petit=sqrt(.Machine$double.eps),r0=10)
     {
petit<-factor*petit
n <- length(eps)
eps.plus<-rep(0,n)
eps.moins<-rep(0,n)
for(i in 1:n){
{if(eps[i]>=0)eps.plus[i]<-log(max(eps[i],petit)^2)}
{if(eps[i]<0)eps.moins[i]<-log(max(-eps[i],petit)^2)}
}
valinit<-c(omega,alpha.plus,alpha.moins,beta)
sig2init<-var(eps[1:min(n,5)])
res <- nlminb(valinit,objf.loggarch,lower=c(-Inf,-Inf,-Inf,-1+petit),
   upper=c(Inf,Inf,Inf,1-petit), eps=eps,n=n,sig2init=sig2init,eps.plus=eps.plus,eps.moins=eps.moins,r0=r0)
omega<-res$par[1]
alpha.plus<-res$par[2]
alpha.moins<-res$par[3]
beta<-res$par[4]
var<-VarAsymp(omega,alpha.plus,alpha.moins,beta,eps,sig2init,petit,r0)
list(coef=res$par,residus=var$residus,var=var$var,log.sig2=var$log.sig2,loglik=res$objective)
}


### HYPERSPHERICAL COORDINATES APPROACH ###

### Construct moving blocks ###

stand<-function(x){
	x2<-x/sqrt(var(x))
}

y.st<-matrix(ncol=m,nrow=n)
for (i in 1:m){
	y.st[,i]<-stand(y[,i])
}

q<-350  
m<-dim(y.st)[2]
s.square<-NULL
sc<-NULL
for (i in 1:n){
	if ((i-(q-1)/2>0) & (i+(q-1)/2<=n)){
	y.block<-y.st[(i-(q-1)/2):(i+(q-1)/2),]
	} else if (i-(q-1)/2<=0){
	y.block<-y.st[1:(i+(q-1)/2),]
	} else if (i+(q-1)/2>n){
	y.block<-y.st[(i-(q-1)/2):n,]
	}
	cov.block<-cov(y.block)
	s.square<-rbind(s.square,diag(cov.block))
	temp<-as.vector(lower.tri(cov.block)*cov.block)
	sc<-rbind(sc,temp[temp!=0])
	}


postscript('trifig1h.eps')
par(mfrow=c(3,1))
for (i in 1:m){
plot(log(s.square[1:(n-1),i]),log(s.square[2:n,i]),xlab=paste('l1s',i,sep=''),ylab=paste('s',i,sep=''),main=paste('series',i,sep=''))
}
dev.off()



s.sq.lag1<-log(s.square[1:(n-1),])
s.sq.lag0<-log(s.square[2:n,])
y.lag1<-log(y[1:(n-1),]^2)

I1=I(y.lag1>0)
I2=I(y.lag1<0)
lambda<-NULL
for (i in 1:m){
start<-lm(s.sq.lag0[,i]~I(I1[,i]*y.lag1[,i])+I(I2[,i]*y.lag1[,i])+s.sq.lag1[,i])
starting<-start$coef
mod.log.var<-estimLogGARCH(starting[1],starting[2],starting[3],starting[4],y[,i],10000)
lambda<-rbind(lambda,as.vector(mod.log.var$coef))
}
lambda.ini<-apply(lambda,2,mean)


############################
##### Newton - Raphson #####
############################

B<-t(chol(cor(y)))
f2<-function(phi){
m<-3   ### CHANGE ACCORDING TO DIMENSION
B<-B
Phi<-matrix(0,m,m)
Phi[lower.tri(Phi,diag=F)]<-phi
Bb<-matrix(NA,nrow=m,ncol=m)
for (i in 1:m){
for (j in 1:m){
if (i==1 & j==1){
Bb[i,j]<-1
} else if (i>=2 & j==1){
Bb[i,j]<-cos(Phi[i,j])
} else if (i==j & i>=2 & i<=m){
Bb[i,j]<-prod(sin(Phi[i,1:(j-1)]))
} else if (j>=2 & j<=i-1){
Bb[i,j]<-cos(Phi[i,j])*prod(sin(Phi[i,1:(j-1)]))
} else if (j>=i+1 & j<=m){
Bb[i,j]<-0
}
}}
 vech(Bb[2:m,2:m])- vech(B[2:m,2:m])
}

phi<-multiroot(f2, start = rep(1,m*(m-1)/2))$root
Phi<-matrix(0,m,m)
Phi[lower.tri(Phi,diag=F)]<-phi

lambda.hat<-lambda.ini
par.hat<-lambda.hat
diff<-100
iter<-1

while (diff>10^(-6)&iter<=200){
	print(c(iter,diff))
	log.var.hat<-matrix(NA,n,m)
	log.var.hat[1,]<-diag(log(var(y)))
	for (i in 2:n){
	for (j in 1:m){
		log.var.hat[i,j]<-lambda.hat[1]+(lambda.hat[2]*(y[i-1,j]>=0)+lambda.hat[3]*(y[i-1,j]<0))*log(y[i-1,j]^2+10^-6)+lambda.hat[4]*log.var.hat[i-1,j]
	}}
	var.hat<-exp(log.var.hat)
	log.var.der1<-log.var.der2<-log.var.der3<-log.var.der4<-matrix(NA,nrow=n,ncol=m)
	log.var.der1[1,]<-log.var.der2[1,]<-log.var.der3[1,]<-log.var.der4[1,]<-rep(0,m)
	for (i in 2:n){
	for (j in 1:m){
		log.var.der1[i,j]<-1+lambda.hat[4]*log.var.der1[i-1,j]
		log.var.der2[i,j]<-(y[i-1,j]>=0)*log(y[i-1,j]^2+10^-6)+lambda.hat[4]*log.var.der2[i-1,j]
		log.var.der3[i,j]<-(y[i-1,j]<0)*log(y[i-1,j]^2+10^-6)+lambda.hat[4]*log.var.der3[i-1,j]
		log.var.der4[i,j]<-log.var.hat[i-1,j]+lambda.hat[4]*log.var.der4[i-1,j]
	}}	
		
	U3.t<-array(NA,dim=c(4,1,n))
	I33.t<-array(NA,dim=c(4,4,n))

	T.hat<-D.hat<-R.hat<-array(NA,dim=c(m,m,n))
	H<-array(NA,dim=c(m,m,n))
	y.vec<-h<-array(NA,dim=c(m,1,n))
	Z.trans<-array(NA,dim=c(4,m,n))


	for (i in 1:n){

		for (k in 1:m){
		for (l in 1:m){
		if (k==1 & l==1){
		T.hat[k,l,i]<-1
		} else if (k>=2 & l==1){
		T.hat[k,l,i]<-cos(Phi[k,l])
		} else if (k==l & k>=2 & k<=m){
		T.hat[k,l,i]<-prod(sin(Phi[k,1:(l-1)]))
		} else if (l>=2 & l<=k-1){
		T.hat[k,l,i]<-cos(Phi[k,l])*prod(sin(Phi[k,1:(l-1)]))
		} else if (l>=k+1 & l<=m){
		T.hat[k,l,i]<-0
		}
		}}
		D.hat[,,i]<-diag(sqrt(var.hat[i,]))+(10^(-6))*diag(m)
		R.hat[,,i]<-T.hat[,,i]%*%t(T.hat[,,i])+(10^(-6))*diag(m)
		y.vec[,,i]<-as.matrix(y[i,],ncol=1)
		H[,,i]<-solve(R.hat[,,i])%*%solve(D.hat[,,i])%*%y.vec[,,i]%*%t(y.vec[,,i])%*%solve(D.hat[,,i])
		h[,,i]<-diag(H[,,i])
		for (j in 1:m){
			Z.trans[1,j,i]<-log.var.der1[i,j]
			Z.trans[2,j,i]<-log.var.der2[i,j]
			Z.trans[3,j,i]<-log.var.der3[i,j]
			Z.trans[4,j,i]<-log.var.der4[i,j]						
		}
		U3.t[,,i]<-Z.trans[,,i]%*%(h[,,i]-matrix(rep(1,m),ncol=1))
		I33.t[,,i]<-Z.trans[,,i]%*%(diag(m)+solve(R.hat[,,i])*R.hat[,,i])%*%t(Z.trans[,,i])
	
	}		

	U3<-matrix(rep(0,4),nrow=4,ncol=1)
	I33<-matrix(rep(0,16),nrow=4,ncol=4)
	for (i in 1:n){
		U3<-U3+0.5*U3.t[,,i]
		I33<-I33+I33.t[,,i]
	}
	par.new<-par.hat+solve(I33)%*%U3
	iter<-iter+1
	diff<-abs(norm(par.new-par.hat))
	par.hat<-par.new
	lambda.hat<-par.new
}

lambda.hat

#### Estimated variances and covariances ####

log.var.hyper<-matrix(NA,nrow=n,ncol=m)
log.var.hyper[1,]<-diag(log(var(y)))
for (i in 2:n){
for (j in 1:m){
	log.var.hyper[i,j]<-lambda.hat[1]+(lambda.hat[2]*(y[i-1,j]>=0)+lambda.hat[3]*(y[i-1,j]<0))*log(y[i-1,j]^2+10^-6)+lambda.hat[4]*log.var.hyper[i-1,j]
}}
var.hyper<-exp(log.var.hyper)

S.hyper<-array(NA,dim=c(m,m,n))
Rho<-cor(y)
for (i in 1:n){
S.hyper[,,i]<-diag(sqrt(var.hyper[i,]))%*%Rho%*%diag(sqrt(var.hyper[i,]))
}



### CHOLESKY DECOMPOSITION APPROACH###

### Construct moving blocks ###

q<-350  
m<-dim(y)[2]
s.square<-NULL
sc<-NULL
S.mw<-array(NA,dim=c(m,m,n))
d.sq.mw<-matrix(NA,n,m-1)
for (i in 1:n){
	if ((i-(q-1)/2>0) & (i+(q-1)/2<=n)){
	y.block<-y[(i-(q-1)/2):(i+(q-1)/2),]
	} else if (i-(q-1)/2<=0){
	y.block<-y[1:(i+(q-1)/2),]
	} else if (i+(q-1)/2>n){
	y.block<-y[(i-(q-1)/2):n,]
	}
	cov.block<-cov(y.block)
	S.mw[,,i]<-cov.block
	s.square<-rbind(s.square,diag(cov.block))
	temp<-as.vector(lower.tri(cov.block)*cov.block)
	sc<-rbind(sc,temp[temp!=0])
	for (j in 2:m){
		mod.mw<-lm(y.block[,j]~y.block[,1:(j-1)]-1)
		d.sq.mw[i,j-1]<-var(mod.mw$residuals)
	}
}
d.sq.mw<-cbind(s.square[,1],d.sq.mw)


postscript('trifig1.eps')
par(mfrow=c(3,1))
for (i in 1:m){
plot(log(d.sq.mw[1:(n-1),i]),log(d.sq.mw[2:n,i]),xlab=paste('l1d',i,sep=''),ylab=paste('d',i,sep=''),main=paste('s',i,sep=''))
}
dev.off()


### Regression models to obtain phi's ###

e<-matrix(NA,nrow=n,ncol=m)
phi.chol<-NULL
e[,1]<-y[,1]
for (i in 2:m){
mod.res<-lm(y[,i]~e[,1:(i-1)]-1)
e[,i]<-mod.res$residuals
phi.chol<-c(phi.chol,mod.res$coef)
}
T.chol<-diag(m)
T.chol[lower.tri(T.chol,diag=F)]<- -phi.chol


### Fit log-GARCH models ###

s.sq.lag1<-log(d.sq.mw[1:(n-1),])
s.sq.lag0<-log(d.sq.mw[2:n,])
e.lag1<-log(e[1:(n-1),]^2+10^-6)
I1=I(e.lag1>=0)
I2=I(e.lag1<0)
lambda<-NULL
log.d.sq<-NULL
for (j in 1:m){
	start=lm(s.sq.lag0[,j]~I(I1[,j]*e.lag1[,j])+I(I2[,j]*e.lag1[,j])+s.sq.lag1[,j])
	starting=start$coef
	fit1<-estimLogGARCH(starting[1],starting[2],starting[3],starting[4],e[,j],10000)
	lambda<-rbind(lambda,fit1$coef)
	log.d.sq<-cbind(log.d.sq,fit1$log.sig2)
}
	d.sq<-exp(log.d.sq)


S.chol<-array(NA,dim=c(m,m,n))
for (i in 1:n){
S.chol[,,i]<-solve(T.chol)%*%diag(d.sq[i,])%*%solve(t(T.chol))
}


### Starting values for the CCC (DCC)-GARCH model ###
	
obs<-e^2
starting.garch<-NULL
#starting.garch1<-NULL
#starting.garch2<-NULL
for (i in 1:m){
start.garch=lm(s.square[2:n,i]~obs[1:(n-1),]+s.square[1:(n-1),])
starting.garch<-rbind(starting.garch,abs(start.garch$coef))
}
a<- starting.garch[,1]
A<-starting.garch[,2:(m+1)]
B<-starting.garch[,(m+2):(2*m+1)]
R<-cor(y)
dcc.par<-matrix(c(0.4,0.4),nrow=2)


### Fit a CCC-GARCH model to the multivariate series ###

ccc.garch<-eccc.estimation(a, A, B, R, dvar=y, model='diagonal', method="BFGS")
# names(ccc.garch)
S.ccc<-array(NA,dim=c(m,m,n))
for (i in 1:n){
	S.ccc[,,i]<-sqrt(diag(ccc.garch$h[i,]))%*%ccc.garch$para.mat$R%*%sqrt(diag(ccc.garch$h[i,]))
}


ccc.garch2<-eccc.estimation(a, A, B, R, dvar=y, model='extended', method="BFGS")
# names(ccc.garch2)
S.ccc2<-array(NA,dim=c(m,m,n))
for (i in 1:n){
	S.ccc2[,,i]<-sqrt(diag(ccc.garch2$h[i,]))%*%ccc.garch2$para.mat$R%*%sqrt(diag(ccc.garch2$h[i,]))
}

### Fit a DCC-GARCH model to the multivariate series ###

dcc.garch<-dcc.estimation(a, A, B, dcc.par, dvar=y, model='diagonal', method="BFGS")
# names(dcc.garch)
S.dcc<-array(NA,dim=c(m,m,n))
for (i in 1:n){
	S.dcc[,,i]<-sqrt(diag(dcc.garch$h[i,]))%*%matrix(dcc.garch$DCC[i,],nrow=m)%*%sqrt(diag(dcc.garch$h[i,]))
}

dcc.garch2<-dcc.estimation(a, A, B, dcc.par, dvar=y, model='extended', method="BFGS")
# names(dcc.garch2)
S.dcc2<-array(NA,dim=c(m,m,n))
for (i in 1:n){
	S.dcc2[,,i]<-sqrt(diag(dcc.garch2$h[i,]))%*%matrix(dcc.garch2$DCC[i,],nrow=m)%*%sqrt(diag(dcc.garch2$h[i,]))
}



### Loss measures ###

D1<-D2<-D3<-D4<-matrix(NA,nrow=n,ncol=6)
MAE<-MSE<-matrix(NA,nrow=n,ncol=6)
colnames(D1)<-colnames(D2)<-colnames(D3)<-colnames(D4)<-c('hyper','chol','ccc','dcc','ccc2','dcc2')

for (i in 1:n){

# Entropy loss
D1[i,1]<- sum(diag(solve(S.mw[,,i])%*%S.hyper[,,i]))-log(det(solve(S.mw[,,i])%*%S.hyper[,,i]))-m
D1[i,2]<- sum(diag(solve(S.mw[,,i])%*%S.chol[,,i]))-log(det(solve(S.mw[,,i])%*%S.chol[,,i]))-m
D1[i,3]<- sum(diag(solve(S.mw[,,i])%*%S.ccc[,,i]))-log(det(solve(S.mw[,,i])%*%S.ccc[,,i]))-m
D1[i,4]<- sum(diag(solve(S.mw[,,i])%*%S.dcc[,,i]))-log(det(solve(S.mw[,,i])%*%S.dcc[,,i]))-m
D1[i,5]<- sum(diag(solve(S.mw[,,i])%*%S.ccc2[,,i]))-log(det(solve(S.mw[,,i])%*%S.ccc2[,,i]))-m
D1[i,6]<- sum(diag(solve(S.mw[,,i])%*%S.dcc2[,,i]))-log(det(solve(S.mw[,,i])%*%S.dcc2[,,i]))-m

# Kullback-Leibler loss
D2[i,1]<- sum(diag(solve(S.hyper[,,i])%*%S.mw[,,i]))-log(det(solve(S.hyper[,,i])%*%S.mw[,,i]))-m
D2[i,2]<- sum(diag(solve(S.chol[,,i])%*%S.mw[,,i]))-log(det(solve(S.chol[,,i])%*%S.mw[,,i]))-m
D2[i,3]<- sum(diag(solve(S.ccc[,,i])%*%S.mw[,,i]))-log(det(solve(S.ccc[,,i])%*%S.mw[,,i]))-m
D2[i,4]<- sum(diag(solve(S.dcc[,,i])%*%S.mw[,,i]))-log(det(solve(S.dcc[,,i])%*%S.mw[,,i]))-m
D2[i,5]<- sum(diag(solve(S.ccc2[,,i])%*%S.mw[,,i]))-log(det(solve(S.ccc2[,,i])%*%S.mw[,,i]))-m
D2[i,6]<- sum(diag(solve(S.dcc2[,,i])%*%S.mw[,,i]))-log(det(solve(S.dcc2[,,i])%*%S.mw[,,i]))-m

# Quadratic loss functions
D3[i,1]<-sum(solve(S.mw[,,i])%*%S.hyper[,,i]-diag(m))^2
D3[i,2]<-sum(solve(S.mw[,,i])%*%S.chol[,,i]-diag(m))^2
D3[i,3]<-sum(solve(S.mw[,,i])%*%S.ccc[,,i]-diag(m))^2
D3[i,4]<-sum(solve(S.mw[,,i])%*%S.dcc[,,i]-diag(m))^2
D3[i,5]<-sum(solve(S.mw[,,i])%*%S.ccc2[,,i]-diag(m))^2
D3[i,6]<-sum(solve(S.mw[,,i])%*%S.dcc2[,,i]-diag(m))^2

D4[i,1]<-sum(solve(S.hyper[,,i])%*%S.mw[,,i]-diag(m))^2
D4[i,2]<-sum(solve(S.chol[,,i])%*%S.mw[,,i]-diag(m))^2
D4[i,3]<-sum(solve(S.ccc[,,i])%*%S.mw[,,i]-diag(m))^2
D4[i,4]<-sum(solve(S.dcc[,,i])%*%S.mw[,,i]-diag(m))^2
D4[i,5]<-sum(solve(S.ccc2[,,i])%*%S.mw[,,i]-diag(m))^2
D4[i,6]<-sum(solve(S.dcc2[,,i])%*%S.mw[,,i]-diag(m))^2

# Mean Absolute Deviation
MAE[i,1]<-sum(abs(S.hyper[,,i]-S.mw[,,i]))/(m^2)
MAE[i,2]<-sum(abs(S.chol[,,i]-S.mw[,,i]))/(m^2)
MAE[i,3]<-sum(abs(S.ccc[,,i]-S.mw[,,i]))/(m^2)
MAE[i,4]<-sum(abs(S.dcc[,,i]-S.mw[,,i]))/(m^2)
MAE[i,5]<-sum(abs(S.ccc2[,,i]-S.mw[,,i]))/(m^2)
MAE[i,6]<-sum(abs(S.dcc2[,,i]-S.mw[,,i]))/(m^2)

#Mean Square Error
MSE[i,1]<-sum(abs(S.hyper[,,i]-S.mw[,,i])^2)/(m^2)
MSE[i,2]<-sum(abs(S.chol[,,i]-S.mw[,,i])^2)/(m^2)
MSE[i,3]<-sum(abs(S.ccc[,,i]-S.mw[,,i])^2)/(m^2)
MSE[i,4]<-sum(abs(S.dcc[,,i]-S.mw[,,i])^2)/(m^2)
MSE[i,5]<-sum(abs(S.ccc2[,,i]-S.mw[,,i])^2)/(m^2)
MSE[i,6]<-sum(abs(S.dcc2[,,i]-S.mw[,,i])^2)/(m^2)
}

cbind(apply(D1,2,mean),sqrt(apply(D1,2,var)))
cbind(apply(D2,2,mean),sqrt(apply(D2,2,var)))
cbind(apply(D3,2,mean),sqrt(apply(D3,2,var)))
cbind(apply(D4,2,mean),sqrt(apply(D4,2,var)))
cbind(apply(MAE,2,mean),sqrt(apply(MAE,2,var)))
cbind(apply(MSE,2,mean),sqrt(apply(MSE,2,var)))


Var.mw<-Var.chol<-Var.hyper<-Var.ccc<-Var.dcc<-Var.ccc2<-Var.dcc2<-matrix(NA,nrow=n,ncol=m)
Cov.mw<-Cov.chol<-Cov.hyper<-Cov.ccc<-Cov.dcc<-Cov.ccc2<-Cov.dcc2<-matrix(NA,nrow=n,ncol=m)

for (i in 1:n){
Cov.mw[i,1]<-S.mw[1,2,i]
Cov.chol[i,1]<-S.chol[1,2,i]
Cov.hyper[i,1]<-S.hyper[1,2,i]
Cov.ccc[i,1]<-S.ccc[1,2,i]
Cov.dcc[i,1]<-S.dcc[1,2,i]
Cov.ccc2[i,1]<-S.ccc2[1,2,i]
Cov.dcc2[i,1]<-S.dcc2[1,2,i]
Cov.mw[i,2]<-S.mw[1,3,i]
Cov.chol[i,2]<-S.chol[1,3,i]
Cov.hyper[i,2]<-S.hyper[1,3,i]
Cov.ccc[i,2]<-S.ccc[1,3,i]
Cov.dcc[i,2]<-S.dcc[1,3,i]
Cov.ccc2[i,2]<-S.ccc2[1,3,i]
Cov.dcc2[i,2]<-S.dcc2[1,3,i]
Cov.mw[i,3]<-S.mw[2,3,i]
Cov.chol[i,3]<-S.chol[2,3,i]
Cov.hyper[i,3]<-S.hyper[2,3,i]
Cov.ccc[i,3]<-S.ccc[2,3,i]
Cov.dcc[i,3]<-S.dcc[2,3,i]
Cov.ccc2[i,3]<-S.ccc2[2,3,i]
Cov.dcc2[i,3]<-S.dcc2[2,3,i]

for (j in 1:m){
Var.mw[i,j]<-S.mw[j,j,i]
Var.chol[i,j]<-S.chol[j,j,i]
Var.hyper[i,j]<-S.hyper[j,j,i]
Var.ccc[i,j]<-S.ccc[j,j,i]
Var.dcc[i,j]<-S.dcc[j,j,i]
Var.ccc2[i,j]<-S.ccc2[j,j,i]
Var.dcc2[i,j]<-S.dcc2[j,j,i]
}}


postscript('trifig3.eps')

par(oma = c(4, 1, 1, 1))
par(mfrow=c(3,3))
for (i in 1){
plot(Var.mw[,i],type='l',xlab='time',ylim=range(Var.mw[,i],Var.chol[,i],Var.hyper[,i],Var.ccc[,i],Var.dcc[,i],Var.ccc2[,i],Var.dcc2[,i]),ylab='s1',lwd=3)
lines(Var.chol[,i],col='blue3',lwd=1)
lines(Var.hyper[,i],col='red3',lwd=1)
lines(Var.ccc[,i],col='orange',lwd=1)
lines(Var.dcc[,i],col='yellow',lwd=1)
lines(Var.ccc2[,i],col='purple3',lwd=1)
lines(Var.dcc2[,i],col='green3',lwd=1)
plot.new()
plot.new()
plot(Cov.mw[,i],type='l',xlab='time',ylim=range(Cov.mw[,i],Cov.chol[,i],Cov.hyper[,i],Cov.ccc[,i],Cov.dcc[,i],Cov.ccc2[,i],Cov.dcc2[,i]),ylab='s12',lwd=3)
lines(Cov.chol[,i],col='blue3',lwd=1)
lines(Cov.hyper[,i],col='red3',lwd=1)
lines(Cov.ccc[,i],col='orange',lwd=1)
lines(Cov.dcc[,i],col='yellow',lwd=1)
lines(Cov.ccc2[,i],col='purple3',lwd=1)
lines(Cov.dcc2[,i],col='green3',lwd=1)
}
for (i in 2){
plot(Var.mw[,i],type='l',xlab='time',ylim=range(Var.mw[,i],Var.chol[,i],Var.hyper[,i],Var.ccc[,i],Var.dcc[,i],Var.ccc2[,i],Var.dcc2[,i]),ylab='s2',lwd=3)
lines(Var.chol[,i],col='blue3',lwd=1)
lines(Var.hyper[,i],col='red3',lwd=1)
lines(Var.ccc[,i],col='orange',lwd=1)
lines(Var.dcc[,i],col='yellow',lwd=1)
lines(Var.ccc2[,i],col='purple3',lwd=1)
lines(Var.dcc2[,i],col='green3',lwd=1)
plot.new()
plot(Cov.mw[,i],type='l',xlab='time',ylim=range(Cov.mw[,i],Cov.chol[,i],Cov.hyper[,i],Cov.ccc[,i],Cov.dcc[,i],Cov.ccc2[,i],Cov.dcc2[,i]),ylab='s13',lwd=3)
lines(Cov.chol[,i],col='blue3',lwd=1)
lines(Cov.hyper[,i],col='red3',lwd=1)
lines(Cov.ccc[,i],col='orange',lwd=1)
lines(Cov.dcc[,i],col='yellow',lwd=1)
lines(Cov.ccc2[,i],col='purple3',lwd=1)
lines(Cov.dcc2[,i],col='green3',lwd=1)
}
for (i in 3){
plot(Cov.mw[,i],type='l',xlab='time',ylim=range(Cov.mw[,i],Cov.chol[,i],Cov.hyper[,i],Cov.ccc[,i],Cov.dcc[,i],Cov.ccc2[,i],Cov.dcc2[,i]),ylab='s23',lwd=3)
lines(Cov.chol[,i],col='blue3',lwd=1)
lines(Cov.hyper[,i],col='red3',lwd=1)
lines(Cov.ccc[,i],col='orange',lwd=1)
lines(Cov.dcc[,i],col='yellow',lwd=1)
lines(Cov.ccc2[,i],col='purple3',lwd=1)
lines(Cov.dcc2[,i],col='green3',lwd=1)
plot(Var.mw[,i],type='l',xlab='time',ylim=range(Var.mw[,i],Var.chol[,i],Var.hyper[,i],Var.ccc[,i],Var.dcc[,i],Var.ccc2[,i],Var.dcc2[,i]),ylab='s3',lwd=3)
lines(Var.chol[,i],col='blue3',lwd=1)
lines(Var.hyper[,i],col='red3',lwd=1)
lines(Var.ccc[,i],col='orange',lwd=1)
lines(Var.dcc[,i],col='yellow',lwd=1)
lines(Var.ccc2[,i],col='purple3',lwd=1)
lines(Var.dcc2[,i],col='green3',lwd=1)
}
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("Moving blocks","Cholesky","Hyperspherical","diagonal CCC","diagonal DCC","extended CCC","extended DCC"), xpd = TRUE, horiz = TRUE, inset = c(0, 
    0), bty = "n", lty = c(1, 1, 1, 1,1, 1,1), lwd=c(3, 1, 1, 1,1, 1,1), col = c('black','blue3','red3','orange','yellow','purple3','green3'), cex = 1)
dev.off()
